//
//  LADocumentSynchronizationHandler.h
//  LAAdditions
//
//  Created by Cyril Anger on 19/01/13.
//  Copyright (c) 2013 LyricApps. All rights reserved.
//

#import <Foundation/Foundation.h>

#import <LAAdditions/LAObject.h>
#import <LAAdditions/LAManagedObjectContextSynchronizationSession.h>
#import <LAAdditions/LAImageAdditions.h>

typedef enum {
	LADocumentVersionStatusIdle,
	LADocumentVersionStatusGettingData,
	LADocumentVersionStatusSynchronizing,
	LADocumentVersionStatusError,
	LADocumentVersionStatusCanceled
} LADocumentVersionStatus;

@interface LADocumentVersion : NSObject <LAObject>

@property (strong, nonatomic, readonly) NSString *title;
@property (strong, nonatomic, readonly) NSDate *modificationDate;
@property (getter = isConflict, nonatomic, readonly) BOOL conflict;
@property (nonatomic) LADocumentVersionStatus status;
@property (strong, nonatomic) NSString *error;
@property (getter = isDownloading, nonatomic, readonly) BOOL downloading;
@property (nonatomic, readonly) CGFloat downloadProgress;
@property (weak, nonatomic) LAManagedObjectContextSynchronizationSession *synchronizationSession;

- (id)initWithTitle:(NSString *)title modification:(NSDate *)modificationDate conflict:(BOOL)conflict;
- (void)getDataWithCompletionHandler:(void (^)(NSURL *URL, BOOL deleteWhenDone, NSError *error))completionHandler progressHandler:(void (^)(CGFloat progress))progressHandler;
- (void)createDocumentWithCompletionHandler:(void (^)(id document, NSError *error))completionHandler progressHandler:(void (^)(CGFloat progress))progressHandler;
- (void)resolveConflict;

@end

@interface LADocumentSynchronizationHandler : NSObject

@property (weak, nonatomic) id document;
@property (nonatomic, readonly) id identifier;
@property (nonatomic, readonly) BOOL canSynchronize;
@property (getter = isActive) BOOL active;
@property (getter = isUploading, nonatomic, readonly) BOOL uploading;
@property (nonatomic, readonly) CGFloat uploadProgress;
@property (getter = isDownloading, nonatomic, readonly) BOOL downloading;
@property (nonatomic, readonly) CGFloat downloadProgress;
@property (getter = isUploadingOrDownloading, nonatomic, readonly) BOOL uploadingOrDownloading;
@property (nonatomic, readonly) CGFloat progress;
@property (strong) NSDate *lastSynchronizationDate;
@property (strong) NSArray *versionsToSynchronize;
@property (strong, readonly) NSOperationQueue *queue;

+ (NSString *)description;
+ (LAImage *)icon;
+ (void)searchAvailableDocumentsWithCompletionHandler:(void (^)(NSArray *availableDocuments))completionHandler;
+ (void)showLoginController;

- (void)reset;
- (NSDictionary *)saveSettings;
- (void)loadSettings:(NSDictionary *)settings;
- (BOOL)handleArchiveURL:(NSURL *)archiveURL error:(NSError * __autoreleasing *)error;
- (void)updateVersionsToSynchronize;
- (void)validateSynchronizationWithVersion:(LADocumentVersion *)version;

@end
