//
//  LAOutlineView.h
//  LAAdditions
//
//  Created by Cyril Anger on 06/03/11.
//  Copyright 2011 LyricApps. All rights reserved.
//

#import <Foundation/Foundation.h>

#import <LAAdditions/LAColumn.h>

extern NSString *LAOutlineViewSizeDidChangeNotification;

@interface LAOutlineView : NSOutlineView <NSMenuDelegate>

@property (nonatomic, readonly) BOOL allowsCustomization;
@property (nonatomic) BOOL makeFirstResponderOnFirstClick;
@property (strong, nonatomic) LAColor *rowBackgroundColor;
@property (strong, nonatomic) LAColor *alternateRowBackgroundColor;
@property (nonatomic, readonly) NSArray *visibleTableColumns;

- (void)noteHeightOfRowsChanged;
- (void)scrollRowIndexesToVisible:(NSIndexSet *)indexes;
- (void)restoreExpandedItems;
- (void)saveEditedItem;
- (void)restoreEditedItem;

- (IBAction)expandAll:(id)sender;
- (IBAction)collapseAll:(id)sender;

@end

@protocol LAOutlineViewDataSource <NSOutlineViewDataSource>

@optional

- (void)outlineView:(NSOutlineView *)outlineView deleteItems:(NSArray *)items;
- (void)outlineView:(NSOutlineView *)outlineView draggingSession:(NSDraggingSession *)session willBeginAtPoint:(NSPoint)screenPoint;
- (void)outlineView:(NSOutlineView *)outlineView draggingSession:(NSDraggingSession *)session movedToPoint:(NSPoint)screenPoint;
- (void)outlineView:(NSOutlineView *)outlineView draggingSession:(NSDraggingSession *)session endedAtPoint:(NSPoint)screenPoint operation:(NSDragOperation)operation;;

@end

@interface NSArray (LATableColumnAdditions)

- (NSArray *)duplicatedTableColumns;
- (BOOL)sizeTableColumnsToFitWidth:(CGFloat)width withIntercolumnSpacing:(CGFloat)intercolumnSpacing;

@end

@interface LATableColumn : NSTableColumn

@property (getter = isHideable, nonatomic) BOOL hideable;
@property (strong, nonatomic) id representedObject;

@end

@interface LATableHeaderView : NSTableHeaderView

@property (strong, nonatomic) NSArray *columnsSets;
@property (nonatomic, readonly) CGFloat lineHeight;

- (LAColumnsSet *)columnsSetNamed:(NSString *)name;
- (void)drawBackgroundInRect:(LARect)rect;
- (void)drawSeparatorInRect:(LARect)rect;
- (void)drawColumn:(LAColumn *)column inRect:(LARect)rect;
- (void)drawColumnTitle:(NSString *)title withAlignment:(NSTextAlignment)alignment inRect:(LARect)rect;

@end
