//
//  LATreeNodesDataSource.h
//  LAAdditions
//
//  Created by Cyril Anger on 27/03/11.
//  Copyright 2011 LyricApps. All rights reserved.
//

#import <LAAdditions/LATreeNode.h>
#import <LAAdditions/LAObserver.h>
#import <LAAdditions/LATableRowView.h>

@class LAFiltersBar;

typedef void (^LATreeNodesDataSourceBlock)(LATreeNodesDataSource *dataSource, LATreeNode *item);

@interface LATreeNodesDataSource : NSObject <LAObserverDelegate>

@property (weak, nonatomic) LATreeNode *parentNode;
@property (strong, nonatomic, readonly) NSString *name;
@property (nonatomic, readonly) NSString *fullID;
@property (nonatomic) BOOL hidden;
@property (nonatomic, readonly) NSArray *registeredNodes;
@property (strong, nonatomic, readonly) NSMutableArray *nodes;
@property (strong, nonatomic, readonly) NSArray *rows;
@property (nonatomic, readonly) BOOL updatesSiblings;
@property (strong, nonatomic) LAObserver *observer;
@property (strong, nonatomic) NSManagedObjectContext *managedObjectContext;
@property (strong, nonatomic) NSString *objectsEntityName;
@property (strong, nonatomic) NSString *groupsEntityName;
@property (weak, nonatomic) Class objectsClass;
@property (weak, nonatomic) Class groupsClass;
@property (nonatomic) BOOL canSelect;
@property (nonatomic) BOOL canSelectMultipleNodes;
@property (strong, nonatomic) NSArray *selectedNodes;
@property (strong, nonatomic) NSArray *selectedObjects;
@property (nonatomic, readonly) NSArray *visibleCells;
@property (nonatomic) BOOL canEdit;
@property (nonatomic, readonly) BOOL canAdd;
@property (nonatomic, readonly) BOOL canAddGroup;
@property (nonatomic, readonly) BOOL canRemove;
@property (nonatomic) BOOL deleteObjectsOnRemove;
@property (nonatomic) BOOL showsRemoveConfirmation;
@property (nonatomic) BOOL canBeOrdered;
@property (nonatomic, readonly) BOOL canOrder;
@property (nonatomic, readonly) NSString *objectsPasteboardType;
@property (nonatomic, readonly) NSArray *copyPasteboardTypes;
@property (nonatomic, readonly) NSArray *pastePasteboardTypes;
@property (nonatomic, readonly) NSArray *dragPasteboardTypes;
@property (weak, nonatomic) LATreeNodesDataSource *detailsDataSource;
@property (copy, nonatomic) LATreeNodesDataSourceBlock doubleClickHandler;
@property (nonatomic) BOOL callDoubleClickHandlerOnAddNewObject;
@property (nonatomic, readonly) BOOL shouldUpdateTreeView;
@property (nonatomic, readonly) NSArray *sortKeyPaths;
@property (copy, nonatomic) NSString *searchString;

@property (weak, nonatomic) IBOutlet LAFiltersBar *filtersBar;

+ (Class)nodeClass;
+ (LATreeNodesDataSource *)dataSourceWithNode:(LATreeNode *)node;

- (id)initWithName:(NSString *)name;
- (LATreeNode *)createNodeWithRepresentedObject:(id <LAObject>)object;
- (LATreeNode *)nodeWithID:(NSString *)ID;
- (void)registerNodes:(NSArray *)nodes;
- (void)unregisterNodes:(NSArray *)nodes;
- (void)insertNodes:(NSArray *)nodes atIndexes:(NSIndexSet *)indexes;
- (void)removeNodesAtIndexes:(NSIndexSet *)indexes;
- (void)computeRows;
- (BOOL)invalidateRows;
- (void)observedObjectsDidChange;
- (void)selectAllNodes;
- (id)objectForItem:(LATreeNode *)item;
- (CGFloat)heightOfRowByItem:(LATreeNode *)item;
- (LATableCellView *)viewForItem:(LATreeNode *)item withIdentifier:(NSString *)identifier;
- (LATableRowView *)rowViewForItem:(LATreeNode *)item;
- (BOOL)shouldShowOutlineCellForItem:(LATreeNode *)item;
- (void)reloadData;
- (id)newObject;
- (id)newGroup;
- (void)insertObjects:(NSArray *)objects inParent:(id)parent atIndexes:(NSIndexSet *)indexes;
- (void)addObjects:(NSArray *)objects;
- (void)addNewObject:(id <LAObject>)newObject;
- (void)removeObjects:(NSArray *)objects forMoving:(BOOL)moving;
- (void)copyObjectsToPasteboard:(NSArray *)objects;
- (void)pasteObjectsFromPasteboard;
- (BOOL)writeItems:(NSArray *)items toPasteboard:(NSPasteboard *)pasteboard;
- (NSPasteboardItem *)pasteboardItemForItem:(LATreeNode *)item;
- (NSArray *)draggedItems:(id <NSDraggingInfo>)info;
- (NSDragOperation)validateDrop:(id <NSDraggingInfo>)info forItem:(LATreeNode *)item childIndex:(NSInteger)index;
- (BOOL)acceptDrop:(id <NSDraggingInfo>)info forItem:(LATreeNode *)item childIndex:(NSInteger)index;
- (NSArray *)namesOfPromisedFilesDroppedAtDestination:(NSURL *)dropDestination forDraggedItems:(NSArray *)items;
- (void)enableTreeViewUpdates;
- (void)disableTreeViewUpdates;
- (void)updateSortDescriptors:(NSArray *)sortDescriptors;
- (void)saveSortDescriptors;
- (void)restoreSortDescriptors;
- (void)sortDescriptorsDidChange;
- (void)updateFilterPredicate;

- (IBAction)add:(id)sender;
- (IBAction)addGroup:(id)sender;
- (IBAction)remove:(id)sender;
- (IBAction)editSortDescriptors:(id)sender;

@end

@interface LAStandardTreeNodesDataSource : LATreeNodesDataSource

@end
